@@ -12,14 +12,29 @@ c_code:	part(s) {1}
 part:	  comment
 	| function_definition
 	{
-	 my $function = $item[1]->[0];
-	 push @{$thisparser->{data}->{functions}}, $function;
-	 $thisparser->{data}->{function}->{$function}->{return_type} = 
-             $item[1]->[1];
-	 $thisparser->{data}->{function}->{$function}->{arg_types} = 
-             [map {ref $_ ? $_->[0] : '...'} @{$item[1]->[2]}];
-	 $thisparser->{data}->{function}->{$function}->{arg_names} = 
-             [map {ref $_ ? $_->[1] : '...'} @{$item[1]->[2]}];
+	 my $function = $item[1][0];
+         $return = 1, last if $thisparser->{data}{done}{$function}++;
+	 push @{$thisparser->{data}{functions}}, $function;
+	 $thisparser->{data}{function}{$function}{return_type} = 
+             $item[1][1];
+	 $thisparser->{data}{function}{$function}{arg_types} = 
+             [map {ref $_ ? $_->[0] : '...'} @{$item[1][2]}];
+	 $thisparser->{data}{function}{$function}{arg_names} = 
+             [map {ref $_ ? $_->[1] : '...'} @{$item[1][2]}];
+	}
+	| function_declaration
+	{
+         $return = 1, last unless $thisparser->{data}{AUTOWRAP};
+	 my $function = $item[1][0];
+         $return = 1, last if $thisparser->{data}{done}{$function}++;
+         my $dummy = 'arg1';
+	 push @{$thisparser->{data}{functions}}, $function;
+	 $thisparser->{data}{function}{$function}{return_type} = 
+             $item[1][1];
+         $thisparser->{data}{function}{$function}{arg_types} = 
+             [map {ref $_ ? $_->[0] : '...'} @{$item[1][2]}];
+	 $thisparser->{data}{function}{$function}{arg_names} = 
+             [map {ref $_ ? ($_->[1] || $dummy++) : '...'} @{$item[1][2]}];
 	}
 	| anything_else
 
@@ -30,6 +45,10 @@ function_definition:
 	rtype IDENTIFIER '(' <leftop: arg ',' arg>(s?) ')' '{'
 	{[@item[2,1], $item[4]]}
 
+function_declaration:
+	rtype IDENTIFIER '(' <leftop: arg_decl ',' arg_decl>(s?) ')' ';'
+	{[@item[2,1], $item[4]]}
+
 rtype:  TYPE star(s?)
         {
          $return = $item[1];
@@ -48,6 +67,9 @@ rtype:  TYPE star(s?)
 arg:	  type IDENTIFIER {[@item[1,2]]}
 	| '...'
 
+arg_decl: type IDENTIFIER(s?) {[$item[1], $item[2][0] || '']}
+	| '...'
+
 type:   TYPE star(s?)
         {
          $return = $item[1];
