@@ -2,7 +2,7 @@ package Inline;
 
 use strict;
 require 5.005;
-$Inline::VERSION = '0.40';
+$Inline::VERSION = '0.41';
 
 use AutoLoader 'AUTOLOAD';
 use Inline::denter;
@@ -206,6 +206,7 @@ my $lexwarn = ($] >= 5.006) ? 'no warnings;' : '';
 
 eval <<END;
 $lexwarn
+\$INIT = \$INIT; # Needed by Sarathy's patch.
 sub INIT {
     \$INIT++;
     &init;
@@ -278,7 +279,7 @@ sub glue {
     $o->print_info if $o->{CONFIG}{PRINT_INFO};
     unless ($o->{INLINE}{object_ready}) {
 	$o->build();
-	$o->write_inl_file();
+	$o->write_inl_file() unless $o->{CONFIG}{_INSTALL_};
     }
     if ($o->{INLINE}{ILSM_suffix} ne 'so' and
 	$o->{INLINE}{ILSM_suffix} ne 'dll' and
@@ -297,6 +298,7 @@ sub glue {
 #==============================================================================
 my (%DATA, %DATA_read);
 sub read_DATA {
+    require Socket;
     my ($marker, $marker_tag);
     my $o = shift;
     my ($pkg, $language_id) = @{$o->{API}}{qw(pkg language_id)};
@@ -304,7 +306,9 @@ sub read_DATA {
 	no strict 'refs';
 	*Inline::DATA = *{$pkg . '::DATA'};
 	local ($/);
-	@{$DATA{$pkg}} = split /(?m)(__\S+?__\n)/, <Inline::DATA>;
+	my ($CR, $LF) = (&Socket::CR, &Socket::LF);
+	(my $data = <Inline::DATA>) =~ s/$CR?$LF/\n/g;
+	@{$DATA{$pkg}} = split /(?m)(__\S+?__\n)/, $data;
 	shift @{$DATA{$pkg}} unless ($ {$DATA{$pkg}}[0] || '') =~ /__\S+?__\n/;
     }
     ($marker, $o->{API}{code}) = splice @{$DATA{$pkg}}, 0, 2;
@@ -405,36 +409,31 @@ sub check_installed {
     my @modparts = split(/::/,$o->{API}{module});
     $o->{API}{modfname} = $modparts[-1];
     $o->{API}{modpname} = join('/',@modparts);
-    if ($realpath =~ m|^blib[/\\]lib$| ) {
-	$realpath = 'blib/arch';
-    }
-    else {
-	croak M29_error_relative_path($o->{CONFIG}{NAME}, $realpath)
-	  unless $realpath =~ m{^(/|[a-zA-Z]:[/\\])};
-    }
+    $realpath =~ s|\bblib[/\\]lib$|blib/arch|;
     
-    my $inl = "$realpath/auto/$o->{API}{modpname}/$o->{API}{modfname}.inl";
-    croak M30_error_no_inl($o->{CONFIG}{NAME}, $o->{API}{pkg}, 
-			   $realpath) unless -f $inl;
-    my %inl;
-    {   local ($/, *INL);
-	open INL, $inl or croak M31_inline_open_failed($inl);
-	%inl = Inline::denter->new()->undent(<INL>);
-    }
-    croak M32_error_md5_validation($o->{INLINE}{md5}, $inl)
-      unless ($o->{INLINE}{md5} eq $inl{md5});
-    croak M33_error_old_inline_version($inl) 
-      unless ($inl{md5} ge '0.40');
-    croak M34_error_incorrect_version($inl) 
-      if $inl{version} ne $o->{API}{version};
-    $o->{API}{location} = "$realpath/auto/$o->{API}{modpname}/" . 
-      "$o->{API}{modfname}.$inl{ILSM}{suffix}";
-    croak M35_error_no_object_file($o->{API}{location}, $inl)
-      unless -f $o->{API}{location};
-    croak M57_wrong_architecture($o->{API}{location}, 
-				 $inl{Config}{archname},
-				 $Config{archname})
-      unless $inl{Config}{archname} eq $Config{archname};
+    my $suffix = ($^O eq 'aix') ? 'so' : $Config{so};
+    my $obj = "$realpath/auto/$o->{API}{modpname}/$o->{API}{modfname}.$suffix";
+    croak M30_error_no_obj($o->{CONFIG}{NAME}, $o->{API}{pkg}, 
+			   $realpath) unless -f $obj;
+#    my %inl;
+#    {   local ($/, *INL);
+#	open INL, $inl or croak M31_inline_open_failed($inl);
+#	%inl = Inline::denter->new()->undent(<INL>);
+#    }
+#    croak M32_error_md5_validation($o->{INLINE}{md5}, $inl)
+#      unless ($o->{INLINE}{md5} eq $inl{md5});
+#    croak M33_error_old_inline_version($inl) 
+#      unless ($inl{md5} ge '0.40');
+#    croak M34_error_incorrect_version($inl) 
+#      if $inl{version} ne $o->{API}{version};
+#    $o->{API}{location} = "$realpath/auto/$o->{API}{modpname}/" . 
+#      "$o->{API}{modfname}.$inl{ILSM}{suffix}";
+#    croak M35_error_no_object_file($o->{API}{location}, $inl)
+#      unless -f $o->{API}{location};
+#    croak M57_wrong_architecture($o->{API}{location}, 
+#				 $inl{Config}{archname},
+#				 $Config{archname})
+#      unless $inl{Config}{archname} eq $Config{archname};
     
     @{$o->{CONFIG}}{qw( PRINT_INFO 
 			REPORTBUG 
@@ -443,9 +442,9 @@ sub check_installed {
 		      )} = (0, 0, 0, 0);
 
     $o->{install_lib} = $realpath;
-    $o->{INLINE}{ILSM_type} = $inl{ILSM}{type};
-    $o->{INLINE}{ILSM_module} = $inl{ILSM}{module};
-    $o->{INLINE}{ILSM_suffix} = $inl{ILSM}{suffix};
+    $o->{INLINE}{ILSM_type} = 'compiled';
+    $o->{INLINE}{ILSM_module} = 'Inline::C';
+    $o->{INLINE}{ILSM_suffix} = $suffix;
     $o->{INLINE}{object_ready} = 1;
 }
 
@@ -454,11 +453,15 @@ sub check_installed {
 #==============================================================================
 sub load {
     my $o = shift;
+
     if ($o->{CONFIG}{_INSTALL_}) {
-	(my $inline = "blib/arch/auto/$o->{API}{pkg}/.inline") =~ s|::|/|g;
+	my $inline = "$o->{API}{modfname}.inl";
 	open INLINE, "> $inline" 
 	  or croak M24_open_for_output_failed($inline);
-	close INLINE; # I::F bug
+	print INLINE "*** AUTOGENERATED by Inline.pm ***\n\n";
+	print INLINE "This file satisfies the make dependency for ";
+	print INLINE "$o->{API}{modfname}.pm\n";
+	close INLINE;
 	return;
     }
 
@@ -710,8 +713,7 @@ sub create_config_file {
 	$inline =~ s|/+|/|g;
 	$inline =~ s|/?Inline\.pm||;
 	$inline ||= '.';
-#	my $INC = "-I$inline -I" . join(" -I", @INC);
-	my $INC = "-I$inline";
+	my $INC = "-I$inline -I" . join(" -I", grep {-d "$_/Inline"} @INC);
 	system "$perl $INC -MInline=_CONFIG_ -e1 $dir"
 	  and croak M20_config_creation_failed($dir);
 	return;
@@ -841,6 +843,8 @@ sub install {
     my ($module, $DIRECTORY);
     my $o = shift;
 
+    croak M64_install_not_c($o->{API}{language_id})
+      unless uc($o->{API}{language_id}) eq 'C';
     croak M36_usage_install_main()
       if ($o->{API}{pkg} eq 'main');
     croak M37_usage_install_auto()
@@ -850,15 +854,16 @@ sub install {
     croak M39_usage_install_version()
       unless $o->{CONFIG}{VERSION};
     croak M40_usage_install_badname($o->{CONFIG}{NAME}, $o->{API}{pkg})
-      unless ($o->{CONFIG}{NAME} eq $o->{API}{pkg} or
-	      $o->{CONFIG}{NAME} =~ /^$o->{API}{pkg}::\w(\w|::)+$/
-	     );
+      unless $o->{CONFIG}{NAME} eq $o->{API}{pkg};
+#	      $o->{CONFIG}{NAME} =~ /^$o->{API}{pkg}::\w(\w|::)+$/
+#	     );
 
     my ($mod_name, $mod_ver, $ext_name, $ext_ver) = 
       ($o->{API}{pkg}, $ARGV[0], @{$o->{CONFIG}}{qw(NAME VERSION)});
     croak M41_usage_install_version_mismatch($mod_name, $mod_ver, 
 					     $ext_name, $ext_ver)
       unless ($mod_ver eq $ext_ver);
+    $o->{INLINE}{INST_ARCHLIB} = $ARGV[1];
 
     $o->{API}{version} = $o->{CONFIG}{VERSION};
     $o->{API}{module} = $o->{CONFIG}{NAME};
@@ -871,7 +876,7 @@ sub install {
 			   );
     $o->{API}{directory} = $o->{INLINE}{DIRECTORY};
     my $cwd = Cwd::cwd();
-    $o->{API}{install_lib} = "$cwd/blib/arch";
+    $o->{API}{install_lib} = "$cwd/$o->{INLINE}{INST_ARCHLIB}";
     $o->{API}{location} = "$o->{API}{install_lib}/auto/" .
       "$o->{API}{modpname}/$o->{API}{modfname}.$o->{INLINE}{ILSM_suffix}";
     unshift @::INC, $o->{API}{install_lib};
@@ -1555,18 +1560,23 @@ sub M29_error_relative_path {
     return <<END;
 Can't load installed extension '$name'
 from relative path '$path'.
+
 END
 #'
 }
 
-sub M30_error_no_inl {
+sub M30_error_no_obj {
     my ($name, $pkg, $path) = @_;
     <<END;
 The extension '$name' is not properly installed in path:
-'$path'
+  '$path'
+
+If this is a CPAN/distributed module, you may need to reinstall it on your
+system.
 
 To allow Inline to compile the module in a temporary cache, simply remove the
 Inline config option 'VERSION=' from the $pkg module.
+
 END
 }
 
@@ -1672,7 +1682,7 @@ sub M40_usage_install_badname {
     my ($name, $pkg) = @_;
     return <<END;
 The NAME '$name' is illegal for this Inline extension.
-The NAME must match or begin with the current package name:
+The NAME must match the current package name:
     $pkg
 
 END
@@ -1910,5 +1920,15 @@ You may need to reinstall this module.
 END
 }
 
+sub M64_install_not_c {
+    my ($lang) = @_;
+    return <<END;
+Invalid attempt to install an Inline module using the '$lang' language.
+
+Only C based modules are currently supported.
+
+END
+}
+
 1;
 __END__
